package pol;

public class PotentialField {
    private final double[] potential;
    private final int width;
    private final int height;

    public PotentialField(final int width, final int height) {
        this.width = width;
        this.height = height;
        potential = new double[width * height];
    }

    public double value(final int x, final int y) {
        return potential[x * height + y];
    }

    public double[] gradient(final int x, final int y) {
        double gx;
        double gy;

        if (x < 1) {
            gx = value(x, y);
        } else {
            gx = value(x, y) - value(x - 1, y);
        }

        if (y < 1) {
            gy = value(x, y);
        } else {
            gy = value(x, y) - value(x, y - 1);
        }

        return new double[]{gx, gy};
    }

    /**
     * Compute the force from the potential field
     * @param x
     * @param y
     * @param vx
     * @param vy
     * @return the new speed, assuming unit mass, time, and everything.
     */
    public double[] dv(final int x, final int y, final double vx, final double vy) {
        final double[] grad = gradient(x, y);
        return new double[]{vx - grad[0], vy - grad[1]};
    }

    /**
     * Add a bell-shaped blob somewhere
     * @param x
     * @param y
     * @param value
     * @param size the standard deviation of the blob
     */
    public void addBlob(final int x, final int y, final double value, final double size) {
        for (int i = 0; i < width; i++) {
            final int mx = i - x;
            for (int j = 0; j < height; j++) {
                final int my = j - y;
                final double v = value * Math.exp(-0.5 * (mx*mx + my*my) / (size*size));
                potential[i * height + j] += v;
            }
        }
    }

    public void clear() {
        for (int i = 0; i < width; i++) {
            for (int j = 0; j < height; j++) {
                potential[i * height + j] = 0.0;
            }
        }
    }

    private double wallPotential(double x, double value, double size) {
        return value / ((x / size + 1) * (x / size + 1));
    }

    private double wallValue(int x, int y, double value, double size) {
        // 4 regions: top, right, bottom, left; defined by two intersecting diagonals
        final double a = height * x - width * y;
        final double b = height * x + width * y - width * height;

        if (a > 0 && b > 0) {  // right
            return wallPotential(width - x, value, size);
        } else if (a > 0 && b <= 0) {  // bottom
            return wallPotential(y, value, size);
        } else if (a <= 0 && b <= 0) {  // left
            return wallPotential(x, value, size);
        } else {
            return wallPotential(height - y, value, size);
        }
    }

    /**
     * Add half-bells ish lines along walls
     */
    public void addWalls(final double value, final double size) {
        for (int i = 0; i < width; i++) {
            for (int j = 0; j < height; j++) {
                potential[i * height + j] += wallValue(i, j, value, size);
            }
        }
    }
}